@@ -16,6 +16,12 @@ def stream_unzip(zipfile_chunks, password=None, chunk_size=65536):
     aes_extra_signature = b'\x01\x99'
     central_directory_signature = b'\x50\x4b\x01\x02'
 
+    def next_or_truncated_error(it):
+        try:
+            return next(it)
+        except StopIteration:
+            raise TruncatedDataError from None
+
     def get_byte_readers(iterable):
         # Return functions to return/"replace" bytes from/to the iterable
         # - _yield_all: yields chunks as they come up (often for a "body")
@@ -48,10 +54,7 @@ def stream_unzip(zipfile_chunks, password=None, chunk_size=65536):
 
             while num:
                 if not chunk:
-                    try:
-                        chunk = next(it)
-                    except StopIteration:
-                        raise ValueError('Fewer bytes than expected in zip') from None
+                    chunk = next_or_truncated_error(it)
                 prev_offset = offset
                 prev_chunk = chunk
                 to_yield = min(num, len(chunk) - offset, chunk_size)
@@ -96,13 +99,19 @@ def stream_unzip(zipfile_chunks, password=None, chunk_size=65536):
     def get_deflate_decompressor():
         dobj = zlib.decompressobj(wbits=-zlib.MAX_WBITS)
 
+        def _decompress_single(compressed_chunk):
+            try:
+                return dobj.decompress(compressed_chunk, chunk_size)
+            except zlib.error as e:
+                raise DeflateError() from e
+
         def _decompress(compressed_chunk):
-            uncompressed_chunk = dobj.decompress(compressed_chunk, chunk_size)
+            uncompressed_chunk = _decompress_single(compressed_chunk)
             if uncompressed_chunk:
                 yield uncompressed_chunk
 
             while dobj.unconsumed_tail and not dobj.eof:
-                uncompressed_chunk = dobj.decompress(dobj.unconsumed_tail, chunk_size)
+                uncompressed_chunk = _decompress_single(dobj.unconsumed_tail)
                 if uncompressed_chunk:
                     yield uncompressed_chunk
 
@@ -160,12 +169,10 @@ def stream_unzip(zipfile_chunks, password=None, chunk_size=65536):
                 update_keys(byte)
 
             if decrypt(get_num(12))[11] != mod_time >> 8:
-                raise ValueError('Incorrect password')
+                raise IncorrectZipCryptoPasswordError()
 
-            for chunk in chunks:
-                yield from decompress(decrypt(chunk))
-                if is_done():
-                    break
+            while not is_done():
+                yield from decompress(decrypt(next_or_truncated_error(chunks)))
 
             return_unused(num_unused())
 
@@ -181,29 +188,27 @@ def stream_unzip(zipfile_chunks, password=None, chunk_size=65536):
 
             keys = PBKDF2(password, salt, 2 * key_length + password_verification_length, 1000)
             if keys[-2:] != password_verification:
-                raise ValueError('Incorrect password')
+                raise IncorrectAESPasswordError()
 
             decrypter = AES.new(
                 keys[:key_length], AES.MODE_CTR,
                 counter=Counter.new(nbits=128, little_endian=True)
             )
             hmac = HMAC.new(keys[key_length:key_length*2], digestmod=SHA1)
-            for chunk in chunks:
+
+            while not is_done():
+                chunk = next_or_truncated_error(chunks)
                 yield from decompress(decrypter.decrypt(chunk))
                 hmac.update(chunk[:len(chunk) - num_unused()])
-                if is_done():
-                    break
 
             return_unused(num_unused())
 
             if get_num(10) != hmac.digest()[:10]:
-                raise ValueError('Invalid MAC')
+                raise HMACIntegrityError()
 
         def no_decrypt_decompress(chunks, decompress, is_done, num_unused):
-            for chunk in chunks:
-                yield from decompress(chunk)
-                if is_done():
-                    break
+            while not is_done():
+                yield from decompress(next_or_truncated_error(chunks))
 
             return_unused(num_unused())
 
@@ -234,7 +239,7 @@ def stream_unzip(zipfile_chunks, password=None, chunk_size=65536):
                 yield chunk
 
             if crc_32_actual != get_crc_32_expected():
-                raise ValueError('CRC-32 does not match')
+                raise CRC32IntegrityError()
 
         version, flags, raw_compression, mod_time, mod_date, crc_32_expected, compressed_size, uncompressed_size, file_name_len, extra_field_len = \
             local_file_header_struct.unpack(get_num(local_file_header_struct.size))
@@ -244,12 +249,12 @@ def stream_unzip(zipfile_chunks, password=None, chunk_size=65536):
 
         flag_bits = tuple(get_flag_bits(flags))
         if (
-            flag_bits[4]      # Enhanced deflate (Deflate64)
+            flag_bits[4]      # Enhanced deflating
             or flag_bits[5]   # Compressed patched
             or flag_bits[6]   # Strong encrypted
             or flag_bits[13]  # Masked header values
         ):
-            raise ValueError('Unsupported flags {}'.format(flag_bits))
+            raise UnsupportedFlagsError(flag_bits)
 
         is_weak_encrypted = flag_bits[0] and raw_compression != 99
         is_aes_encrypted = flag_bits[0] and raw_compression == 99
@@ -260,7 +265,7 @@ def stream_unzip(zipfile_chunks, password=None, chunk_size=65536):
             raw_compression
 
         if compression not in (0, 8):
-            raise ValueError('Unsupported compression type {}'.format(compression))
+            raise UnsupportedCompressionTypeError(compression)
 
         has_data_descriptor = flag_bits[3]
 
@@ -302,4 +307,49 @@ def stream_unzip(zipfile_chunks, password=None, chunk_size=65536):
                 pass
             break
         else:
-            raise ValueError(b'Unexpected signature ' + signature)
+            raise UnexpectedSignatureError(signature)
+
+class UnzipError(ValueError):
+    pass
+
+class DataError(UnzipError):
+    pass
+
+class TruncatedDataError(DataError):
+    pass
+
+class UncompressError(UnzipError):
+    pass
+
+class DeflateError(UncompressError):
+    pass
+
+class UnsupportedFeatureError(DataError):
+    pass
+
+class UnsupportedFlagsError(UnsupportedFeatureError):
+    pass
+
+class UnsupportedCompressionTypeError(UnsupportedFeatureError):
+    pass
+
+class UnexpectedSignatureError(DataError):
+    pass
+
+class IntegrityError(DataError):
+    pass
+
+class HMACIntegrityError(IntegrityError):
+    pass
+
+class CRC32IntegrityError(IntegrityError):
+    pass
+
+class IncorrectPasswordError(UnzipError):
+    pass
+
+class IncorrectZipCryptoPasswordError(IncorrectPasswordError):
+    pass
+
+class IncorrectAESPasswordError(IncorrectPasswordError):
+    pass
